@@ -318,27 +318,6 @@ _cairo_win32_surface_create_similar (void	    *abstract_src,
     return _cairo_win32_surface_create_for_dc (src->dc, format, width, height);
 }
 
-/**
- * _cairo_win32_surface_create_dib:
- * @format: format of pixels in the surface to create 
- * @width: width of the surface, in pixels
- * @height: height of the surface, in pixels
- * 
- * Creates a device-independent-bitmap surface not associated with
- * any particular existing surface or device context. The created
- * bitmap will be unititialized.
- * 
- * Return value: the newly created surface, or %NULL if it couldn't
- *   be created (probably because of lack of memory)
- **/
-cairo_surface_t *
-_cairo_win32_surface_create_dib (cairo_format_t format,
-				 int	        width,
-				 int	        height)
-{
-    return _cairo_win32_surface_create_for_dc (NULL, format, width, height);
-}
-
 static cairo_status_t
 _cairo_win32_surface_finish (void *abstract_surface)
 {
@@ -452,9 +431,9 @@ _cairo_win32_surface_release_source_image (void                   *abstract_surf
 
 static cairo_status_t
 _cairo_win32_surface_acquire_dest_image (void                    *abstract_surface,
-					 cairo_rectangle_t       *interest_rect,
+					 cairo_rectangle_fixed_t *interest_rect,
 					 cairo_image_surface_t  **image_out,
-					 cairo_rectangle_t       *image_rect,
+					 cairo_rectangle_fixed_t *image_rect,
 					 void                   **image_extra)
 {
     cairo_win32_surface_t *surface = abstract_surface;
@@ -517,11 +496,11 @@ _cairo_win32_surface_acquire_dest_image (void                    *abstract_surfa
 }
 
 static void
-_cairo_win32_surface_release_dest_image (void                   *abstract_surface,
-					 cairo_rectangle_t      *interest_rect,
-					 cairo_image_surface_t  *image,
-					 cairo_rectangle_t      *image_rect,
-					 void                   *image_extra)
+_cairo_win32_surface_release_dest_image (void                    *abstract_surface,
+					 cairo_rectangle_fixed_t *interest_rect,
+					 cairo_image_surface_t   *image,
+					 cairo_rectangle_fixed_t *image_rect,
+					 void                    *image_extra)
 {
     cairo_win32_surface_t *surface = abstract_surface;
     cairo_win32_surface_t *local = image_extra;
@@ -632,7 +611,7 @@ _composite_alpha_blend (cairo_win32_surface_t *dst,
 }
 
 static cairo_int_status_t
-_cairo_win32_surface_composite (cairo_operator_t	operator,
+_cairo_win32_surface_composite (cairo_operator_t	op,
 				cairo_pattern_t       	*pattern,
 				cairo_pattern_t		*mask_pattern,
 				void			*abstract_dst,
@@ -652,7 +631,7 @@ _cairo_win32_surface_composite (cairo_operator_t	operator,
     int integer_transform;
     int itx, ity;
 
-    if (pattern->type != CAIRO_PATTERN_SURFACE ||
+    if (pattern->type != CAIRO_PATTERN_TYPE_SURFACE ||
 	pattern->extend != CAIRO_EXTEND_NONE)
 	return CAIRO_INT_STATUS_UNSUPPORTED;
 
@@ -660,7 +639,7 @@ _cairo_win32_surface_composite (cairo_operator_t	operator,
 	/* FIXME: When we fully support RENDER style 4-channel
 	 * masks we need to check r/g/b != 1.0.
 	 */
-	if (mask_pattern->type != CAIRO_PATTERN_SOLID)
+	if (mask_pattern->type != CAIRO_PATTERN_TYPE_SOLID)
 	    return CAIRO_INT_STATUS_UNSUPPORTED;
 
 	alpha = ((cairo_solid_pattern_t *)mask_pattern)->color.alpha_short >> 8;
@@ -707,8 +686,8 @@ _cairo_win32_surface_composite (cairo_operator_t	operator,
 
     if (alpha == 255 &&
 	src->format == dst->format &&
-	(operator == CAIRO_OPERATOR_SOURCE ||
-	 (src->format == CAIRO_FORMAT_RGB24 && operator == CAIRO_OPERATOR_OVER))) {
+	(op == CAIRO_OPERATOR_SOURCE ||
+	 (src->format == CAIRO_FORMAT_RGB24 && op == CAIRO_OPERATOR_OVER))) {
 	
 	if (!BitBlt (dst->dc,
 		     dst_x, dst_y,
@@ -723,7 +702,7 @@ _cairo_win32_surface_composite (cairo_operator_t	operator,
     } else if (integer_transform &&
 	       (src->format == CAIRO_FORMAT_RGB24 || src->format == CAIRO_FORMAT_ARGB32) &&
 	       dst->format == CAIRO_FORMAT_RGB24 &&
-	       operator == CAIRO_OPERATOR_OVER) {
+	       op == CAIRO_OPERATOR_OVER) {
 
 	return _composite_alpha_blend (dst, src, alpha,
 				       src_x, src_y,
@@ -740,7 +719,7 @@ _cairo_win32_surface_composite (cairo_operator_t	operator,
  *       super-luminescent colors (a == 0, r,g,b > 0)
  */
 static enum { DO_CLEAR, DO_SOURCE, DO_NOTHING, DO_UNSUPPORTED }
-categorize_solid_dest_operator (cairo_operator_t operator,
+categorize_solid_dest_operator (cairo_operator_t op,
 				unsigned short   alpha)
 {
     enum { SOURCE_TRANSPARENT, SOURCE_LIGHT, SOURCE_SOLID, SOURCE_OTHER } source;
@@ -752,7 +731,7 @@ categorize_solid_dest_operator (cairo_operator_t operator,
     else
 	source = SOURCE_OTHER;
     
-    switch (operator) {
+    switch (op) {
     case CAIRO_OPERATOR_CLEAR:    /* 0                 0 */
     case CAIRO_OPERATOR_OUT:      /* 1 - Ab            0 */
 	return DO_CLEAR;
@@ -813,9 +792,9 @@ categorize_solid_dest_operator (cairo_operator_t operator,
 
 static cairo_int_status_t
 _cairo_win32_surface_fill_rectangles (void			*abstract_surface,
-				      cairo_operator_t		operator,
+				      cairo_operator_t		op,
 				      const cairo_color_t	*color,
-				      cairo_rectangle_t		*rects,
+				      cairo_rectangle_fixed_t		*rects,
 				      int			num_rects)
 {
     cairo_win32_surface_t *surface = abstract_surface;
@@ -833,7 +812,7 @@ _cairo_win32_surface_fill_rectangles (void			*abstract_surface,
     /* Optimize for no destination alpha (surface->pixman_image is non-NULL for all
      * surfaces with alpha.)
      */
-    switch (categorize_solid_dest_operator (operator, color->alpha_short)) {
+    switch (categorize_solid_dest_operator (op, color->alpha_short)) {
     case DO_CLEAR:
 	new_color = RGB (0, 0, 0);
 	break;	
@@ -966,8 +945,8 @@ _cairo_win32_surface_set_clip_region (void              *abstract_surface,
 }
 
 static cairo_int_status_t
-_cairo_win32_surface_get_extents (void		    *abstract_surface,
-				  cairo_rectangle_t *rectangle)
+_cairo_win32_surface_get_extents (void		          *abstract_surface,
+				  cairo_rectangle_fixed_t *rectangle)
 {
     cairo_win32_surface_t *surface = abstract_surface;
 
@@ -982,6 +961,18 @@ _cairo_win32_surface_flush (void *abstract_surface)
     return _cairo_surface_reset_clip (abstract_surface);
 }
 
+/**
+ * cairo_win32_surface_create:
+ * @hdc: the DC to create a surface for
+ * 
+ * Creates a cairo surface that targets the given DC.  The DC will be
+ * queried for its initial clip extents, and this will be used as the
+ * size of the cairo surface.  Also, if the DC is a raster DC, it will
+ * be queried for its pixel format and the cairo surface format will
+ * be set appropriately.
+ * 
+ * Return value: the newly created surface
+ **/
 cairo_surface_t *
 cairo_win32_surface_create (HDC hdc)
 {
@@ -1005,6 +996,8 @@ cairo_win32_surface_create (HDC hdc)
 	    format = CAIRO_FORMAT_ARGB32;
 	else if (depth == 24)
 	    format = CAIRO_FORMAT_RGB24;
+        else if (depth == 16)
+            format = CAIRO_FORMAT_RGB24;
 	else if (depth == 8)
 	    format = CAIRO_FORMAT_A8;
 	else if (depth == 1)
@@ -1055,6 +1048,28 @@ cairo_win32_surface_create (HDC hdc)
     return (cairo_surface_t *)surface;
 }
 
+/**
+ * cairo_win32_surface_create_with_dib:
+ * @format: format of pixels in the surface to create 
+ * @width: width of the surface, in pixels
+ * @height: height of the surface, in pixels
+ * 
+ * Creates a device-independent-bitmap surface not associated with
+ * any particular existing surface or device context. The created
+ * bitmap will be unititialized.
+ * 
+ * Return value: the newly created surface
+ *
+ **/
+cairo_surface_t *
+cairo_win32_surface_create_with_dib (cairo_format_t format,
+                                     int	    width,
+                                     int	    height)
+{
+    return _cairo_win32_surface_create_for_dc (NULL, format, width, height);
+}
+
+
 /**
  * _cairo_surface_is_win32:
  * @surface: a #cairo_surface_t
@@ -1069,7 +1084,33 @@ _cairo_surface_is_win32 (cairo_surface_t *surface)
     return surface->backend == &cairo_win32_surface_backend;
 }
 
+/**
+ * cairo_win32_surface_get_dc
+ * @surface: a #cairo_surface_t
+ *
+ * Returns the HDC associated with this surface, or NULL if none.
+ * Also returns NULL if the surface is not a win32 surface.
+ *
+ * Return value: HDC or NULL if no HDC available.
+ **/
+HDC
+cairo_win32_surface_get_dc (cairo_surface_t *surface)
+{
+    cairo_win32_surface_t *winsurf;
+
+    if (surface == NULL)
+	return NULL;
+
+    if (!_cairo_surface_is_win32(surface))
+	return NULL;
+
+    winsurf = (cairo_win32_surface_t *) surface;
+
+    return winsurf->dc;
+}
+
 static const cairo_surface_backend_t cairo_win32_surface_backend = {
+    CAIRO_SURFACE_TYPE_WIN32,
     _cairo_win32_surface_create_similar,
     _cairo_win32_surface_finish,
     _cairo_win32_surface_acquire_source_image,
@@ -1085,8 +1126,7 @@ static const cairo_surface_backend_t cairo_win32_surface_backend = {
     _cairo_win32_surface_set_clip_region,
     NULL, /* intersect_clip_path */
     _cairo_win32_surface_get_extents,
-    NULL, /* show_glyphs */
-    NULL, /* fill_path */
+    NULL, /* old_show_glyphs */
     NULL, /* get_font_options */
     _cairo_win32_surface_flush,
     NULL  /* mark_dirty_rectangle */
@@ -1102,7 +1142,6 @@ static const cairo_surface_backend_t cairo_win32_surface_backend = {
 CRITICAL_SECTION cairo_toy_font_face_hash_table_mutex;
 CRITICAL_SECTION cairo_scaled_font_map_mutex;
 CRITICAL_SECTION cairo_ft_unscaled_font_map_mutex;
-CRITICAL_SECTION _global_image_glyph_cache_mutex;
 
 BOOL WINAPI
 DllMain (HINSTANCE hinstDLL,
@@ -1116,13 +1155,11 @@ DllMain (HINSTANCE hinstDLL,
     InitializeCriticalSection (&cairo_toy_font_face_hash_table_mutex);
     InitializeCriticalSection (&cairo_scaled_font_map_mutex);
     InitializeCriticalSection (&cairo_ft_unscaled_font_map_mutex);
-    InitializeCriticalSection (&_global_image_glyph_cache_mutex);
     break;
   case DLL_PROCESS_DETACH:
     DeleteCriticalSection (&cairo_toy_font_face_hash_table_mutex);
     DeleteCriticalSection (&cairo_scaled_font_map_mutex);
     DeleteCriticalSection (&cairo_ft_unscaled_font_map_mutex);
-    DeleteCriticalSection (&_global_image_glyph_cache_mutex);
     break;
   }
   return TRUE;
